// ---------------------------
// bwadmin custom Map Rotation
// ---------------------------
//
// Use this file to change the map rotation Tribes 2 uses, when a map ends this
// list is used to decide the next map.  To add maps or edit maps on this list
// use the following command:
//
//   bwadmin_mapAdd(%file, %type, %min, %max);
//     %file - Filename of the map.
//     %type - Game type.
//     %min  - min num of players required, -1 for unrestricted (optional)
//     %min  - max num of players required, -1 for unrestricted (optional)
//
// eg: bwadmin_mapAdd("Sanctuary", CTF, -1, 32);
//     bwadmin_mapAdd("Fracas", DM);
//
// If you add custom maps to the server you can either add them in by hand or
// use the following function from the server console to rebuild the custom map
// rotation file:
//
//   bwadmin_CreateMapRotationFile();
//     WARNING: This file will be overwritten so make sure you aren't losing any
//     changes.
//
// Notes:
// 1) Any maps not on this list will not be in the rotation or the server menus.
//    You can add custom maps to this list that do not come with Tribes2.
// 2) By default Tribes 2 rotates through the maps of the same type, so when a
//    CTF map ends the next map will be CTF also. There is an option to enable
//    rotation of all maps on the list. Refer to the ServerPrefs documentation.
// 3) Min/max number of players is optional and if not included the map will
//    always be in the rotation.
// 4) You need to add the map for each of the types you need to it played for.
//    For instance SunDried is a DM, Hunters, Bounty and Rabbit map.
// 5) If the map name has spaces in it you must include it in quotes, eg:
//    bwadmin_mapAdd("Custom Map", CTF);
//

//
// --- CTF ---
//
bwadmin_mapAdd("Sanctuary", CTF, -1, 32);
bwadmin_mapAdd("Archipelago", CTF, 16, -1);
bwadmin_mapAdd("BeggarsRun", CTF, -1, 16);
bwadmin_mapAdd("Damnation", CTF, -1, 32);
bwadmin_mapAdd("DeathBirdsFly", CTF, 32, -1);
bwadmin_mapAdd("Desiccator", CTF, 16, -1);
bwadmin_mapAdd("DustToDust", CTF, -1, 24);
bwadmin_mapAdd("Firestorm", CTF, -1, 24);
bwadmin_mapAdd("Katabatic", CTF, -1, 24);
bwadmin_mapAdd("Minotaur", CTF, -1, 24);
bwadmin_mapAdd("Quagmire", CTF, 16, -1);
bwadmin_mapAdd("Recalescence", CTF, 16, -1);
bwadmin_mapAdd("Reversion", CTF, -1, 32);
bwadmin_mapAdd("RiverDance", CTF, 8, 32);
bwadmin_mapAdd("Slapdash", CTF, 16, -1);
bwadmin_mapAdd("ThinIce", CTF, -1, 32);
bwadmin_mapAdd("Tombstone", CTF, 8, 32);

//
// --- CnH ---
//
bwadmin_mapAdd("Abominable", CnH, -1, 32);
bwadmin_mapAdd("AshesToAshes", CnH, 16, -1);
bwadmin_mapAdd("Equinox", CnH, -1, 32);
bwadmin_mapAdd("Firestorm", CnH, -1, 24);
bwadmin_mapAdd("Flashpoint", CnH, 16, -1);
bwadmin_mapAdd("Insalubria", CnH, -1, 32);
bwadmin_mapAdd("JacobsLadder", CnH, -1, 32);
bwadmin_mapAdd("Overreach", CnH, 16, -1);
bwadmin_mapAdd("Sirocco", CnH, 16, -1);

//
// --- DM ---
//
bwadmin_mapAdd("AgentsOfFortune", DM);
bwadmin_mapAdd("Casern_Cavite", DM);
bwadmin_mapAdd("Equinox", DM, -1, 32);
bwadmin_mapAdd("Escalade", DM, 16, -1);
bwadmin_mapAdd("Fracas", DM);
bwadmin_mapAdd("Invictus", DM);
bwadmin_mapAdd("MyrkWood", DM, -1, 16);
bwadmin_mapAdd("Oasis", DM, -1, 16);
bwadmin_mapAdd("Pyroclasm", DM);
bwadmin_mapAdd("Rasp", DM);
bwadmin_mapAdd("SunDried", DM, 16, -1);
bwadmin_mapAdd("Talus", DM);
bwadmin_mapAdd("Underhill", DM, 16, -1);
bwadmin_mapAdd("Whiteout", DM, 16, -1);

//
// --- Hunters ---
//
bwadmin_mapAdd("AgentsOfFortune", Hunters);
bwadmin_mapAdd("Casern_Cavite", Hunters);
bwadmin_mapAdd("DustToDust", Hunters, -1, 32);
bwadmin_mapAdd("Escalade", Hunters, 16, -1);
bwadmin_mapAdd("Fracas", Hunters);
bwadmin_mapAdd("Gehenna", Hunters, 16, -1);
bwadmin_mapAdd("MyrkWood", Hunters, -1, 16);
bwadmin_mapAdd("Rasp", Hunters);
bwadmin_mapAdd("Rimehold", Hunters, 16, -1);
bwadmin_mapAdd("SunDried", Hunters);
bwadmin_mapAdd("Talus", Hunters);
bwadmin_mapAdd("Underhill", Hunters);

//
// --- TeamHunters ---
//
bwadmin_mapAdd("AgentsOfFortune", TeamHunters, -1, 32);
bwadmin_mapAdd("DustToDust", TeamHunters, -1, 32);
bwadmin_mapAdd("Escalade", TeamHunters, 16, -1);
bwadmin_mapAdd("Gehenna", TeamHunters, 16, -1);
bwadmin_mapAdd("Rimehold", TeamHunters);

//
// --- Siege ---
//
bwadmin_mapAdd("Alcatraz", Siege);
bwadmin_mapAdd("Caldera", Siege);
bwadmin_mapAdd("Gauntlet", Siege);
bwadmin_mapAdd("IceBound", Siege);
bwadmin_mapAdd("Masada", Siege, -1, 16);
bwadmin_mapAdd("Respite", Siege);
bwadmin_mapAdd("UltimaThule", Siege);

//
// --- Bounty ---
//
bwadmin_mapAdd("Casern_Cavite", Bounty);
bwadmin_mapAdd("Escalade", Bounty, 16, 32);
bwadmin_mapAdd("Rasp", Bounty, -1, 24);
bwadmin_mapAdd("RiverDance", Bounty);
bwadmin_mapAdd("SunDried", Bounty, 16, -1);
bwadmin_mapAdd("Talus", Bounty, -1, 24);
bwadmin_mapAdd("Underhill", Bounty, -1, 24);
bwadmin_mapAdd("Whiteout", Bounty, 16, -1);

//
// --- Rabbit ---
//
bwadmin_mapAdd("Escalade", Rabbit, 16, -1);
bwadmin_mapAdd("MyrkWood", Rabbit, -1, 24);
bwadmin_mapAdd("SunDried", Rabbit);
