// DisplayName = Capture the Flag - Team Aerial Combat

//--- GAME RULES BEGIN ---
//Capture enemy flag and Prevent enemy from capturing yours
//Score 10 points each time enemy flag is "capped"
//Score 3 points for killing enemy vehicles from a vehicle
//Score 1 point for killing enemy vehicles otherwise
//Lose 3 points for destroying own team vehicles.
//Dont touch the ground...it hurts.
//--- GAME RULES END ---


//=====================================================================================
//	TAC2 - CTF Mission Code					Wizard_TPG - 11/5/2001
//
//	This code includes:
//				Basic CTF Game Code
//				Piloting with flag cause flag to drop
//				Player spawns with repair pack
//
//=====================================================================================


//exec the AI scripts
exec("scripts/aiCTF.cs");

package TAC_CTFGame
{
	function Armor::onMount(%this,%obj,%vehicle,%node)
	{
		//Code to return flag if player
		if (%node == 0)
		{
			if(isObject( Game ))
				Game.PilotWithFlag(%obj);
		}
		else if(%vehicle.getDatablock().getName() $= "TACScoutFlyer")
		{
			Game.HawkeyeWithFlag(%obj);
		}
		Parent::onMount(%this,%obj,%vehicle,%node);
	}

	function Flag::objectiveInit(%data, %flag)
	{
	   if (!%flag.isTeamSkinned)
	   {
		  %pos = %flag.getTransform();
		  %group = %flag.getGroup();
	   }
	   %flag.originalPosition = %flag.getTransform();
	   %flag.isHome = true;
	   %flag.carrier = "";
	   setTargetSkin(%flag.getTarget(), $TeamSkin[%flag.team]);
	   setTargetSensorGroup(%flag.getTarget(), %flag.team);
	   setTargetAlwaysVisMask(%flag.getTarget(), 0x7);
	   setTargetRenderMask(%flag.getTarget(), getTargetRenderMask(%flag.getTarget()) | 0x2);
	   %flag.scopeWhenSensorVisible(true);
	   $flagStatus[%flag.team] = "<At Base>";

	   //Point the flag and stand at each other
	   %group = %flag.getGroup();
	   %count = %group.getCount();
	   %flag.stand = "";
	   for(%i = 0; %i < %count; %i++)
	   {
		  %this = %group.getObject(%i);
		  if((%this.getClassName() !$= "InteriorInstance") && (%this.getClassName() !$= "SimGroup"))
		  {
			 if(%this.getDataBlock().getName() $= "ExteriorFlagStand")
			 {
				%flag.stand = %this;
				%this.flag = %flag;
			 }
		  }
	   }

	   // set the nametag on the target
	   setTargetName(%flag.getTarget(), $teamName[%flag.team]);

	   // create a marker on this guy
	   %flag.waypoint = new MissionMarker() {
		  position = %flag.getTransform();
		  dataBlock = "FlagMarker";
	   };
	   MissionCleanup.add(%flag.waypoint);

	   // create a target for this (there is no MissionMarker::onAdd script call)
	   %target = createTarget(%flag.waypoint, $teamName[%flag.team], "", "", 'Base', %flag.team, 0);
	   setTargetAlwaysVisMask(%target, 0xffffffff);

	   //store the flag in an array
	   $TeamFlag[%flag.team] = %flag;
	}

};


//-- tracking  ---
// .deaths .kills .genDefends .carrierKills .flagDefends .escortAssists .suicides .teamKills .turretKills .flagCaps
// .flagReturns .genDestroys .genRepairs

function TAC_CTFGame::initGameVars(%game)
{
	%game.SCORE_PER_VEHICLE_APC_KILL = 3;  	//enemy team gets this for killing your apc from a vehicle
	%game.SCORE_PER_OTHER_APC_KILL = 1;		//enemy team gets this for killing apc otherwise
	%game.SCORE_PER_OWN_APC_KILL = -3;		//Your team gets this if you tk an apc
	%game.SCORE_PER_BASE_APC_KILL = 0;		//Enemy team gets this if your veihcle is destroyed by enemy base turrets
	%game.SCORE_PER_TURRET_APC_KILL = 1;	//Enemy team gets this for kills by other turrets
	%game.SCORE_PER_CRASH_APC_KILL = -1;	//Your team gets this if you crash an apc
	%game.SCORE_PER_COLLIDE_APC_KILL = 1;	//Your team gets this if you collide with an enemy causing their vehicle to be destroyed

	%game.SCORE_PER_SUICIDE = 0;
	%game.SCORE_PER_TEAMKILL = 0;
	%game.SCORE_PER_DEATH = 0;

	%game.SCORE_PER_KILL = 0;
	%game.SCORE_PER_PLYR_FLAG_CAP = 0;
	%game.SCORE_PER_TEAM_FLAG_CAP = 10;
	%game.SCORE_PER_GEN_DESTROY = 0;
	%game.SCORE_PER_ESCORT_ASSIST = 0;

	%game.SCORE_PER_TURRET_KILL = 0;
	%game.SCORE_PER_FLAG_DEFEND = 0;
	%game.SCORE_PER_CARRIER_KILL = 0;
	%game.SCORE_PER_FLAG_RETURN = 0;
	%game.SCORE_PER_GEN_DEFEND = 0;
	%game.SCORE_PER_GEN_REPAIR = 0;

	%game.FLAG_RETURN_DELAY = 15 * 1000;

	%game.TIME_CONSIDERED_FLAGCARRIER_THREAT = 0;  //after damaging enemy flag carrier
	%game.RADIUS_GEN_DEFENSE = 0;  //meters
	%game.RADIUS_FLAG_DEFENSE = 0;  //meters

	%game.fadeTimeMS = 2000;
	%game.notifyMineDist = 7.5;
}

//--------------------------------------------------------------------------------------------------------------------

function TAC_CTFGame::missionLoadDone(%game)
{
	%game.numTeams = 2;

   //continue with the default function
   DefaultGame::missionLoadDone(%game);

   //reset some globals, just in case...
	$dontScoreTimer[1] = false;
	$dontScoreTimer[2] = false;
}

function TAC_CTFGame::setUpTeams(%game)
{
   DefaultGame::setUpTeams(%game);
   %game.numTeams = 2;
   setSensorGroupCount(%game.numTeams + 1);
}

function TAC_CTFGame::equip(%game, %player)
{
   for(%i =0; %i<$InventoryHudCount; %i++)
      %player.client.setInventoryHudItem($InventoryHudData[%i, itemDataName], 0, 1);

   %player.setInventory(RepairKit,1);
   %player.setInventory(FlareGrenade,6);
   %player.setInventory(Blaster,1);
   %player.setInventory(Chaingun,1);
   %player.setInventory(ChaingunAmmo, 100);
   %player.setInventory(Disc,1);
   %player.setInventory(DiscAmmo, 20);
   %player.setInventory(RepairPack,1);
   %player.setInventory(TargetingLaser, 1);
   %player.weaponCount = 3;
   %player.use("Blaster");
}

function TAC_CTFGame::PilotWithFlag(%game, %player)
{
	// this is called when a player attempts to pilot while holding the flag
	if (!isObject(%player.holdingFlag))
		return;

	%client = %player.client;
	%flag = %player.holdingFlag;
	%game.flagReset(%flag);
	%otherTeam = %flag.team;
	messageClient(%player.client, 'MsgCTFFlagReturned', '\c1You have attempted to pilot and lost the flag.~wfx/misc/flag_return.wav', %client.name, 0, %otherTeam);
    messageTeamExcept(%client, 'MsgCTFFlagReturned', '\c2Teammate %1 attempted to pilot and lost the flag.~wfx/misc/flag_return.wav', %client.name, 0, %otherTeam);
    messageTeam(%otherTeam, 'MsgCTFFlagReturned', '\c2Enemy %1 attempted to pilot and lost the flag.~wfx/misc/flag_return.wav', %client.name, $teamName[%flag.team], %otherTeam);
    messageTeam(0, 'MsgCTFFlagReturned', '\c2%1 returned the %2 flag by attempting to pilot.~wfx/misc/flag_return.wav', %client.name, $teamName[%flag.team], %otherTeam);
}

function TAC_CTFGame::HawkeyeWithFlag(%game, %player)
{
	// this is called when a player attempts to pilot while holding the flag
	if (!isObject(%player.holdingFlag))
		return;

	%client = %player.client;
	%flag = %player.holdingFlag;
	%game.flagReset(%flag);

	%otherTeam = %flag.team;
	messageClient(%player.client, 'MsgCTFFlagReturned', '\c1The flag will not fit in the Hawkeye and has been returned.~wfx/misc/flag_return.wav', %client.name, 0, %otherTeam);
    messageTeamExcept(%client, 'MsgCTFFlagReturned', '\c2Teammate %1 attempted to carry the flag in a hawkeye and lost the flag.~wfx/misc/flag_return.wav', %client.name, 0, %otherTeam);
    messageTeam(%otherTeam, 'MsgCTFFlagReturned', '\c2Enemy %1 attempted to carry the flag in a hawkeye and lost the flag.~wfx/misc/flag_return.wav', %client.name, $teamName[%flag.team], %otherTeam);
    messageTeam(0, 'MsgCTFFlagReturned', '\c2%1 returned the %2 flag by attempting to carry the flag in a hawkeye.~wfx/misc/flag_return.wav', %client.name, $teamName[%flag.team], %otherTeam);
}


function TAC_CTFGame::VehicleKill(%game, %data, %obj)
{
	DefaultGame::VehicleKill(%game, %data, %obj);
}

function TAC_CTFGame::playerTouchFlag(%game, %player, %flag)
{
   %client = %player.client;

   if ((%flag.carrier $= "") && (%player.getState() !$= "Dead"))
   {
		//flag isn't held and has been touched by a live player
      if (%client.team == %flag.team)
			%game.playerTouchOwnFlag(%player, %flag);
		else
			%game.playerTouchEnemyFlag(%player, %flag);
   }
}

function TAC_CTFGame::playerTouchOwnFlag(%game, %player, %flag)
{
   if(%flag.isHome)
	{
		if (%player.holdingFlag !$= "")
			%game.flagCap(%player);
	}
   else
      %game.flagReturn(%flag, %player);

   //call the AI function
   //%game.AIplayerTouchOwnFlag(%player, %flag);
}

function TAC_CTFGame::playerTouchEnemyFlag(%game, %player, %flag)
{
   	%client = %player.client;
   	%player.holdingFlag = %flag;  //%player has this flag
   	%flag.carrier = %player;  //this %flag is carried by %player

   	%player.mountImage(FlagImage, $FlagSlot, true, $teamSkin[%flag.team]);

	if(%player.isPilot())
		%game.PilotWithFlag(%player);


	//only cancel the return timer if the player is in bounds...
	if (!%client.outOfBounds)
	{
	   cancel($FlagReturnTimer[%flag]);
		$FlagReturnTimer[%flag] = "";
	}

   %flag.hide(true);
   %flag.startFade(0, 0, false);
   %flag.isHome = false;
	//$flagStatus[%flag.team] = %client.name;
   if(%flag.stand)
  		%flag.stand.getDataBlock().onFlagTaken(%flag.stand);//animate, if exterior stand

	%nameBase = getField(%client.getAuthInfo(), 0);
	$flagStatus[%flag.team] = %nameBase;
   messageTeamExcept(%client, 'MsgCTFFlagTaken', '\c2Teammate %1 took the %2 flag.~wfx/misc/flag_snatch.wav', %client.name, $teamName[%flag.team], %flag.team, %nameBase);
   messageTeam(%flag.team, 'MsgCTFFlagTaken', '\c2Your flag has been taken by %1!~wfx/misc/flag_taken.wav',%client.name, 0, %flag.team, %nameBase);
   messageClient(%client, 'MsgCTFFlagTaken', '\c2You took the %2 flag.~wfx/misc/flag_snatch.wav', %client.name, $teamName[%flag.team], %flag.team, %nameBase);
	echo("LOG: "@%player@" ("@%client@") took team "@%flag.team@" flag");

   //call the AI function
   //%game.AIplayerTouchEnemyFlag(%player, %flag);

	//if the player is out of bounds, then in 3 seconds, it should be thrown back towards the in bounds area...
	if (%client.outOfBounds)
		%game.schedule(3000, "boundaryLoseFlag", %player);
}

function TAC_CTFGame::playerDroppedFlag(%game, %player)
{
   %client = %player.client;
   %flag = %player.holdingFlag;

   %player.holdingFlag = ""; //player isn't holding a flag anymore
   %flag.carrier = "";	//flag isn't held anymore
	$flagStatus[%flag.team] = "<In the Field>";

   %player.unMountImage($FlagSlot);
   %flag.hide(false); //Does the throwItem function handle this?

   messageTeamExcept(%client, 'MsgCTFFlagDropped', '\c2Teammate %1 dropped the %2 flag.~wfx/misc/flag_drop.wav', %client.name, $teamName[%flag.team], %flag.team);
   messageTeam(%flag.team, 'MsgCTFFlagDropped', '\c2Your flag has been dropped by %1!~wfx/misc/flag_drop.wav', %client.name, 0, %flag.team);
	if(!%player.client.outOfBounds)
	   messageClient(%client, 'MsgCTFFlagDropped', '\c2You dropped the %2 flag.~wfx/misc/flag_drop.wav', 0, $teamName[%flag.team], %flag.team);
	echo("LOG: "@%player@" ("@%client@") dropped team "@%flag.team@" flag");

	//don't duplicate the schedule if there's already one in progress...
	if ($FlagReturnTimer[%flag] <= 0)
	   $FlagReturnTimer[%flag] = %game.schedule(%game.FLAG_RETURN_DELAY - %game.fadeTimeMS, "flagReturnFade", %flag);

   //call the AI function
   //%game.AIplayerDroppedFlag(%player, %flag);
}

function TAC_CTFGame::flagCap(%game, %player)
{
   %client = %player.client;
   %flag = %player.holdingFlag;
	%flag.carrier = "";

   //award points to player and team

   messageTeamExcept(%client, 'MsgCTFFlagCapped', '\c2%1 captured the %2 flag!~wfx/misc/flag_capture.wav', %client.name, $teamName[%flag.team],  %flag.team, %client.team);
   messageTeam(%flag.team, 'MsgCTFFlagCapped', '\c2Your flag was captured by %1.~wfx/misc/flag_lost.wav', %client.name, 0, %flag.team, %client.team);
   messageClient(%client, 'MsgCTFFlagCapped', '\c2You captured the %2 flag!~wfx/misc/flag_capture.wav', 0, $teamName[%flag.team], %flag.team, %client.team);

	echo("LOG: "@%player@" ("@%client@") caps team "@%client.team@" flag");
   %player.holdingFlag = ""; //no longer holding it.
   %player.unMountImage($FlagSlot);
   %game.flagReset(%flag);
   %game.awardScoreFlagCap(%client, %flag);

   //call the AI function
   //%game.AIflagCap(%player, %flag);

	//if this cap didn't end the game, play the announcer...
	if ($missionRunning)
	{
		if ($teamName[%client.team] $= 'Inferno')
		   messageAll("", '~wvoice/announcer/ann.infscores.wav');
		else if ($teamName[%client.team] $= 'Storm')
		   messageAll("", '~wvoice/announcer/ann.stoscores.wav');
	}
}

function TAC_CTFGame::flagReturnFade(%game, %flag)
{
   $FlagReturnTimer[%flag] = %game.schedule(%game.fadeTimeMS, "flagReturn", %flag);
	%flag.startFade(%game.fadeTimeMS, 0, true);
}

function TAC_CTFGame::flagReturn(%game, %flag, %player)
{
   cancel($FlagReturnTimer[%flag]);
	$FlagReturnTimer[%flag] = "";

	if(%flag.team == 1)
		%otherTeam = 2;
	else
		%otherTeam = 1;
   if (%player !$= "")
   {
   	//a player returned it
		%client = %player.client;
		messageTeamExcept(%client, 'MsgCTFFlagReturned', '\c2Teammate %1 returned your flag to base.~wfx/misc/flag_return.wav', %client.name, 0, %flag.team);
		messageTeam(%otherTeam, 'MsgCTFFlagReturned', '\c2Enemy %1 returned the %2 flag.~wfx/misc/flag_return.wav', %client.name, $teamName[%flag.team], %flag.team);
		messageClient(%client, 'MsgCTFFlagReturned', '\c2You returned your flag.~wfx/misc/flag_return.wav', 0, $teamName[%flag.team], %flag.team);
		echo("LOG: "@%player@" ("@%client@") returned team "@%flag.team@" flag");
      %game.awardScoreFlagReturn(%player.client);
   }
   else
   {
   	//returned due to timer
      messageTeam(%otherTeam, 'MsgCTFFlagReturned', '\c2The %2 flag was returned to base.~wfx/misc/flag_return.wav', 0, $teamName[%flag.team], %flag.team);  //because it was dropped for too long
  	   messageTeam(%flag.team, 'MsgCTFFlagReturned', '\c2Your flag was returned.~wfx/misc/flag_return.wav', 0, 0, %flag.team);
		echo("LOG: team "@%flag.team@" flag returned (timeout)");
   }
   %game.flagReset(%flag);
}

function TAC_CTFGame::flagReset(%game, %flag)
{
	//make sure if there's a player carrying it (probably one out of bounds...), it is stripped first
	if (isObject(%flag.carrier))
	{
	   %flag.carrier.holdingFlag = ""; //no longer holding it.
	   %flag.carrier.unMountImage($FlagSlot);
	}

	//fades, restore default position, home, velocity, general status, etc.
	%flag.setVelocity("0 0 0");
	%flag.setTransform(%flag.originalPosition);
	%flag.isHome = true;
	%flag.carrier = "";
	$flagStatus[%flag.team] = "<At Base>";
	%flag.hide(false);
	if(%flag.stand)
		%flag.stand.getDataBlock().onFlagReturn(%flag.stand);//animate, if exterior stand

	//fade the flag in...
	%flag.startFade(%game.fadeTimeMS, 0, false);

	//call the AI function
	//%game.AIflagReset(%flag);
}

function TAC_CTFGame::timeLimitReached(%game)
{
	echo("LOG: game over (timelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_CTFGame::scoreLimitReached(%game)
{
	echo("LOG: game over (scorelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_CTFGame::notifyMineDeployed(%game, %mine)
{
	//see if the mine is within 5 meters of the flag stand...
	%mineTeam = %mine.sourceObject.team;
	%homeFlag = $TeamFlag[%mineTeam];
	if (isObject(%homeFlag))
	{
	   %dist = VectorDist(%homeFlag.originalPosition, %mine.position);
		if (%dist <= %game.notifyMineDist)
		{
		   messageTeam(%mineTeam, 'MsgCTFFlagMined', "The flag has been mined.~wvoice/announcer/flag_minedFem.wav" );
		}
	}
}

function TAC_CTFGame::gameOver(%game)
{
	//send the winner message
	%winner = "";
	if ($teamScore[1] > $teamScore[2])
		%winner = $teamName[1];
	else if ($teamScore[2] > $teamScore[1])
		%winner = $teamName[2];

	if (%winner $= 'Storm')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.stowins.wav" );
	else if (%winner $= 'Inferno')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.infwins.wav" );
	else
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.gameover.wav" );

	//call the default
	DefaultGame::gameOver(%game);

   %game.sendDebriefing();
	messageAll('MsgClearObjHud', "");
	for(%i = 0; %i < ClientGroup.getCount(); %i ++)
	{
		%client = %group.getObject(%i);
		%game.resetScore(%client);
	}
   for(%j = 1; %j <= %game.numTeams; %j++)
		$TeamScore[%j] = 0;
}


////////////////////////////////////////////////////////////////////////////////////////
function TAC_CTFGame::clientMissionDropReady(%game, %client)
{
   messageClient(%client, 'MsgClientReady',"", %game.class);

   for(%i = 1; %i <= %game.numTeams; %i++)
   {
      messageClient(%client, 'MsgCTFAddTeam', "", %i, $TeamName[%i], $flagStatus[%i], $TeamScore[%i]);
   }

	messageClient(%client, 'MsgMissionDropInfo', '\c0You are in mission %1 (%2).', $MissionDisplayName, $MissionTypeDisplayName, $ServerName );

	DefaultGame::clientMissionDropReady(%game, %client);
}


function TAC_CTFGame::assignClientTeam(%game, %client, %respawn)
{
	DefaultGame::assignClientTeam(%game, %client, %respawn);
	// if player's team is not on top of objective hud, switch lines
	messageClient(%client, 'MsgCheckTeamLines', "", %client.team);
}

function TAC_CTFGame::updateKillScores(%game, %clVictim, %clKiller, %damageType, %implement)
{
	if(%clKiller <= 0 || %clKiller $= "")
	{
		%game.awardScoreDeath(%clVictim);
	}
	else if (%game.testKill(%clVictim, %clKiller)) //verify victim was an enemy
	{
		%game.awardScoreKill(%clKiller);
		%game.awardScoreDeath(%clVictim);
	}
	else
	{
		if (%game.testSuicide(%clVictim, %clKiller, %damageType))  //otherwise test for suicide
		{
			%game.awardScoreSuicide(%clVictim);
		}
		else
		{
			if (%game.testTeamKill(%clVictim, %clKiller)) //otherwise test for a teamkill
				%game.awardScoreTeamKill(%clVictim, %clKiller);
		}
	}
}

function TAC_CTFGame::awardScoreFlagCap(%game, %cl, %flag)
{
	%cl.flagCaps++;
	$TeamScore[%cl.team] += %game.SCORE_PER_TEAM_FLAG_CAP;
  	messageAll('MsgTeamScoreIs', "", %cl.team, $TeamScore[%cl.team]);

   if (%game.SCORE_PER_TEAM_FLAG_CAP > 0)
   {
	   %plural = (%game.SCORE_PER_PLYR_FLAG_CAP != 1 ? 's' : "");
	   messageClient(%cl, 'msgCTFFriendCap', '\c0Your team received %1 point%2 for capturing the enemy flag!', %game.SCORE_PER_TEAM_FLAG_CAP, %plural);
	   messageTeamExcept(%cl, 'msgCTFFriendCap', '\c0Your team recieved %2 point%3 because %1 captured the enemy flag!', %cl.name, %game.SCORE_PER_TEAM_FLAG_CAP, %plural);
	   messageTeam(%flag.team, 'msgCTFEnemyCap', '\c0Enemy team received %2 point%3 because %1 captured your flag!', %cl.name, %game.SCORE_PER_TEAM_FLAG_CAP, %plural);
   }
   %game.recalcScore(%cl);
   %game.checkScoreLimit(%cl.team);
}

function TAC_CTFGame::checkScoreLimit(%game, %team)
{
	echo("*** current score = ",$TeamScore[1]," - ",$TeamScore[2]);
	%scoreLimit = MissionGroup.TAC_CTF_scoreLimit;
	// default of 100 if scoreLimit not defined
	if(%scoreLimit $= "")
		%scoreLimit = 100;
   if($TeamScore[%team] >= %scoreLimit)
      %game.scoreLimitReached();
}

function TAC_CTFGame::enterMissionArea(%game, %playerData, %player)
{
   if(%player.getState() $= "Dead")
      return;

   	%player.client.outOfBounds = false;
   	messageClient(%player.client, 'EnterMissionArea', '\c1You are back in the mission area.');
	//the instant a player leaves the mission boundary, the flag is dropped, and the return is scheduled...
	if (%player.holdingFlag > 0)
	{
		cancel($FlagReturnTimer[%player.holdingFlag]);
		$FlagReturnTimer[%player.holdingFlag] = "";
	}
}

function TAC_CTFGame::leaveMissionArea(%game, %playerData, %player)
{
   if(%player.getState() $= "Dead")
      return;

	// maybe we'll do this just in case
   	%player.client.outOfBounds = true;
	// if the player is holding a flag, strip it and throw it back into the mission area
	// otherwise, just print a message
	if(%player.holdingFlag > 0)
		%game.boundaryLoseFlag(%player);
	else
      messageClient(%player.client, 'MsgLeaveMissionArea', '\c1You have left the mission area.');
	echo("LOG: "@%player@" ("@%player.client@") leave mission area");
}

function TAC_CTFGame::boundaryLoseFlag(%game, %player)
{
	// this is called when a player goes out of the mission area while holding
	// the enemy flag. - make sure the player is still out of bounds
	if (!%player.client.outOfBounds || !isObject(%player.holdingFlag))
		return;

	%client = %player.client;
	%flag = %player.holdingFlag;
	%flag.setVelocity("0 0 0");
	%flag.setTransform(%player.getWorldBoxCenter());
	%flag.setCollisionTimeout(%player);

	%game.playerDroppedFlag(%player);

	// now for the tricky part -- throwing the flag back into the mission area
	// let's try throwing it back towards its "home"
	%home = %flag.originalPosition;
	%vecx =  firstWord(%home) - firstWord(%player.getWorldBoxCenter());
	%vecy = getWord(%home, 1) - getWord(%player.getWorldBoxCenter(), 1);
	%vecz = getWord(%home, 2) - getWord(%player.getWorldBoxCenter(), 2);
	%vec = %vecx SPC %vecy SPC %vecz;

	// normalize the vector, scale it, and add an extra "upwards" component
	%vecNorm = VectorNormalize(%vec);
	%vec = VectorScale(%vecNorm, 1500);
	%vec = vectorAdd(%vec, "0 0 500");

	// apply the impulse to the flag object
	%flag.applyImpulse(%player.getWorldBoxCenter(), %vec);

	//don't forget to send the message
	messageClient(%player.client, 'MsgCTFFlagDropped', '\c1You have left the mission area and lost the flag.~wfx/misc/flag_drop.wav', 0, 0, %player.holdingFlag.team);
}

function TAC_CTFGame::dropFlag(%game, %player)
{
	if(%player.holdingFlag > 0)
	{
		if (!%player.client.outOfBounds)
	      %player.throwObject(%player.holdingFlag);
		else
			%game.boundaryLoseFlag(%player);
	}
}

function TAC_CTFGame::applyConcussion(%game, %player)
{
   %game.dropFlag( %player );
}

