// DisplayName = Capture and Hold - Team Aerial Combat

//--- GAME RULES BEGIN ---
//A team scores 2 points per second it holds an objective
//Turrets and inventory stations convert when their switch is taken
//Score 3 points for killing enemy vehicles from a vehicle
//Score 1 point for killing enemy vehicles otherwise
//Lose 3 points for destroying own team vehicles.
//The ground hurts...a lot
//--- GAME RULES END ---

//exec the AI scripts
exec("scripts/aiCnH.cs");


package TAC_CnHGame
{
	function FlipFlop::playerTouch(%data, %flipflop, %player)
	{
	   if(%flipflop.team != %player.client.team)
	   {
			Parent::playerTouch(%data, %flipflop, %player);
			Game.startTimerPlayerFFCap(%player.client, %flipflop);
	   }
	}

	function Flipflop::objectiveInit(%data, %flipflop)
	{
	   %flipflop.teamBonusThread = "";
	   %flipflop.playerBonusThread = "";
	   Parent::objectiveInit(%data, %flipflop);
	}
};


//--------- TAC CnH SCORING INIT ------------------
function TAC_CnHGame::initGameVars(%game)
{
	%game.SCORE_PER_VEHICLE_APC_KILL = 3;
	%game.SCORE_PER_OTHER_APC_KILL = 1;
	%game.SCORE_PER_OWN_APC_KILL = -3;
	%game.SCORE_PER_BASE_APC_KILL = 0;
	%game.SCORE_PER_TURRET_APC_KILL = 1;
	%game.SCORE_PER_CRASH_APC_KILL = -1;
	%game.SCORE_PER_COLLIDE_APC_KILL = 1;

	%game.SCORE_PER_SUICIDE = 0;
	%game.SCORE_PER_TEAMKILL = 0;
	%game.SCORE_PER_DEATH = 0;

	%game.SCORE_PER_KILL = 0;
	%game.SCORE_PER_PLYR_FLIPFLOP_CAP = 0;
	%game.SCORE_PER_TEAM_FLIPFLOP_CAP = 1;
	%game.SCORE_PER_TEAM_FLIPFLOP_HOLD = 1;

	%game.SCORE_PER_TURRET_KILL = 0;
	%game.SCORE_PER_FLIPFLOP_DEFEND = 0;
	%game.SCORE_LIMIT_PER_TOWER = 1200; //default of 1500 points per tower required to win @ 1 pt per %game.TIME_REQ_TEAM_HOLD_BONUS milliseconds
	// default of 1500 per switch if not defined in mission file specifically. 2 pts per sec per switch = 10 minute min & 20 minute max mission length
	// not counting time spent waiting for first point(s).  Should be 15-30ish minute missions depending on turnovers

	%game.TIME_REQ_PLYR_CAP_BONUS =	12 * 1000;  //player must hold a switch 12 seconds to get a point for it.
	%game.TIME_REQ_TEAM_CAP_BONUS = 12 * 1000;	//time after touching it takes for team to get a point
	%game.TIME_REQ_TEAM_HOLD_BONUS = 0.5 * 1000;	 //recurring time it takes team to earn a point when flipflop held
	%game.RADIUS_FLIPFLOP_DEFENSE = 20; //meters
}

function TAC_CnHGame::VehicleKill(%game, %data, %obj)
{
	DefaultGame::VehicleKill(%game, %data, %obj);
}

function TAC_CnHGame::setUpTeams(%game)
{

   	DefaultGame::setUpTeams(%game);
   	// reset the visibility of team 0 (team is still defaulted as friendly)
   	setSensorGroupAlwaysVisMask(0, 0);
	%game.numTeams = 2;
   	setSensorGroupCount(%game.numTeams + 1);
}

//-- tracking  ---
// .deaths .kills .suicides .teamKills .turretKills
// .flipFlopDefends .flipFlopsCapped

function TAC_CnHGame::startMatch(%game)
{
   for(%i = 0; %i <= %game.numTeams; %i++)
		$TeamScore[%i] = 0;

   DefaultGame::startMatch(%game);
}

function TAC_CnHGame::checkScoreLimit(%game, %team)
{
	%scoreLimit = %game.getScoreLimit();
	if($TeamScore[%team] >= %scoreLimit)
		%game.scoreLimitReached();
}

function TAC_CnHGame::getScoreLimit(%game)
{
	%scoreLimit = MissionGroup.TAC_CnH_scoreLimit;
	if(%scoreLimit $= "")
		%scoreLimit = %game.getNumFlipFlops() * %game.SCORE_LIMIT_PER_TOWER;

	return %scoreLimit;
}

function TAC_CnHGame::scoreLimitReached(%game)
{
	logEcho("game over (scorelimit)");
	%game.gameOver();
	schedule( 5000, %game, cycleMissions );
}

function TAC_CnHGame::timeLimitReached(%game)
{
	logEcho("game over (timelimit)");
	%game.gameOver();
	schedule( 5000, %game, cycleMissions );
}

function TAC_CnHGame::gameOver(%game)
{
	// stop all bonus timers
	%game.stopScoreTimers();

	//send the winner message
	%winner = "";
	if ($teamScore[1] > $teamScore[2])
		%winner = $teamName[1];
	else if ($teamScore[2] > $teamScore[1])
		%winner = $teamName[2];

	if (%winner $= 'Storm')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.stowins.wav" );
	else if (%winner $= 'Inferno')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.infwins.wav" );
	else
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.gameover.wav" );

	//call the default
	DefaultGame::gameOver(%game);

	messageAll('MsgClearObjHud', "");
	for(%i = 0; %i < ClientGroup.getCount(); %i ++)
	{
		%client = ClientGroup.getObject(%i);
		%game.resetScore(%client);
	}
	for ( %team = 1; %team <= %game.numTeams; %team++ )
		$TeamScore[%team] = 0;
}

function TAC_CnHGame::stopScoreTimers(%game)
{
	// find all switches and cancel any timers associated with them
	%ffGroup = nameToId("MissionCleanup/FlipFlops");
	if(%ffGroup <= 0)
		return;

	for(%i = 0; %i < %ffGroup.getCount(); %i++)
	{
		%curFF = %ffGroup.getObject(%i);
		cancel(%curFF.pCapThread);
		cancel(%curFF.tCapThread);
		cancel(%curFF.tHoldThread);
	}
}

function TAC_CnHGame::clientMissionDropReady(%game, %client)
{
   messageClient(%client, 'MsgClientReady',"", %game.class);
	%scoreLimit = %game.getScoreLimit();

   for(%i = 1; %i <= %game.numTeams; %i++)
	{
		%teamHeld = %game.countFlipsHeld(%i);
      messageClient(%client, 'MsgCnHAddTeam', "", %i, $TeamName[%i], $TeamScore[%i], %scoreLimit, %teamHeld);
	}

	//%game.populateTeamRankArray(%client);
	//messageClient(%client, 'MsgYourRankIs', "", -1);

   messageClient(%client, 'MsgMissionDropInfo', '\c0You are in mission %1 (%2).', $MissionDisplayName, $MissionTypeDisplayName, $ServerName );

	DefaultGame::clientMissionDropReady(%game, %client);
}

function TAC_CnHGame::assignClientTeam(%game, %client, %respawn)
{
	DefaultGame::assignClientTeam(%game, %client, %respawn);
	// if player's team is not on top of objective hud, switch lines
	messageClient(%client, 'MsgCheckTeamLines', "", %client.team);
}

function TAC_CnHGame::getNumFlipFlops()
{
     %ffGroup = nameToID("MissionCleanup/FlipFlops");
	 return %ffGroup.getCount();
}

function TAC_CnHGame::countFlipsHeld(%game, %team)
{
	%teamHeld = 0;
	// count how many flipflops each team holds
	%ffSet = nameToID("MissionCleanup/FlipFlops");
	if(%ffSet > 0)
	{
		%numFF = %ffSet.getCount();
		for(%j = 0; %j < %numFF; %j++)
		{
			%curFF = %ffSet.getObject(%j);
			if(%curFF.team == %team)
				%teamHeld++;
		}
	}

	return %teamHeld;
}

function TAC_CnHGame::countFlips(%game)
{
	return true;
}

function TAC_CnHGame::equip(%game, %player)
{
	for(%i =0; %i<$InventoryHudCount; %i++)
		%player.client.setInventoryHudItem($InventoryHudData[%i, itemDataName], 0, 1);

	//%player.setArmor("Light");
	%player.setInventory(Blaster,1);
	%player.setInventory(Chaingun, 1);
	%player.setInventory(ChaingunAmmo, 100);
	%player.setInventory(Disc,1);
	%player.setInventory(DiscAmmo, 20);
	%player.setInventory(TargetingLaser, 1);
	%player.setInventory(FlareGrenade,6);
	%player.setInventory(Beacon, 3);
   	%player.setInventory(RepairPack,1);
	%player.setInventory(RepairKit,1);
	%player.weaponCount = 3;

	%player.use("Blaster");
}


function TAC_CnHGame::recalcScore(%game, %cl)
{
	%killValue = %cl.kills * %game.SCORE_PER_KILL;
	%deathValue = %cl.deaths * %game.SCORE_PER_DEATH;

	if (%killValue - %deathValue == 0)
		%killPoints = 0;
	else
		%killPoints = (%killValue * %killValue) / (%killValue - %deathValue);

	%cl.offenseScore = %killPoints;
	%cl.offenseScore +=	%cl.suicides * %game.SCORE_PER_SUICIDE + //-1
	%cl.offenseScore +=	%cl.teamKills * %game.SCORE_PER_TEAMKILL; // -1
	%cl.offenseScore += %cl.flipFlopsCapped * %game.SCORE_PER_PLYR_FLIPFLOP_CAP;

	%cl.defenseScore =	%cl.turretKills * %game.SCORE_PER_TURRET_KILL;  // 1
	%cl.defenseScore += %cl.flipFlopDefends *  %game.SCORE_PER_FLIPFLOP_DEFEND;

	%cl.score =	mFloor(%cl.offenseScore + %cl.defenseScore);
	//track switches held (not touched), switches defended, kills, deaths, suicides, tks

	%game.recalcTeamRanks(%cl);
}

function TAC_CnHGame::updateKillScores(%game, %clVictim, %clKiller, %damageType, %implement)
{
	if(%clKiller <= 0 || %clKiller $= "")
	{
		%game.awardScoreDeath(%clVictim);
	}
	else if (%game.testKill(%clVictim, %clKiller)) //verify victim was an enemy
	{
		%game.awardScoreKill(%clKiller);
		%game.awardScoreDeath(%clVictim);
	}
	else
	{
		if (%game.testSuicide(%clVictim, %clKiller, %damageType))  //otherwise test for suicide
		{
			%game.awardScoreSuicide(%clVictim);
		}
		else
		{
			if (%game.testTeamKill(%clVictim, %clKiller)) //otherwise test for a teamkill
				%game.awardScoreTeamKill(%clVictim, %clKiller);
		}
	}
}

function TAC_CnHGame::testPlayerFFDefend(%game, %victimID, %killerID)
{
   InitContainerRadiusSearch(%victimID.plyrPointOfDeath, %game.RADIUS_FLIPFLOP_DEFENSE, $TypeMasks::ItemObjectType);
   %objID = containerSearchNext();
   while(%objID != 0)
   {
	  %objType = %objID.getDataBlock().getName();
	  echo("tFD  found " @ %objType @ " belonging to team " @ %objID.team);
	  if ((%objType $= "FlipFlop") && (%objID.team == %killerID.team))
	   	 return true;  //found a killer's team flipflop near the point of victim's death
	  else
	     %objID = containerSearchNext();
   }
   return false; //didn't find a qualifying flipflop within required radius of victims point of death
}

function TAC_CnHGame::awardScorePlayerFFDefend(%game, %cl, %this)
{
	%cl.flipFlopDefends++;
	//if (%game.SCORE_PER_FLIPFLOP_DEFEND != 0)
      //messageClient(%cl, $scoreFliDefMsg, 'You receive a %1 point bonus for defending the %2.', %game.SCORE_PER_FLIPFLOP_DEFEND, %game.cleanWord(%this.name));
    %game.recalcScore(%cl);
}

function TAC_CnHGame::awardScorePlayerFFCap(%game, %cl, %this)
{
	%cl.flipFlopsCapped++;
    %game.recalcScore(%cl);
}

function TAC_CnHGame::awardScoreTeamFFCap(%game, %team, %this)
{
	cancel(%this.tCapThread);
	$TeamScore[%team] +=%game.SCORE_PER_TEAM_FLIPFLOP_CAP;
	%sLimit = %game.getScoreLimit();
	if (%game.SCORE_PER_TEAM_FLIPFLOP_CAP)
		messageAll('MsgCnHTeamCap', "", -1, -1, -1, %team, $teamScore[%team], %sLimit);
   if (%game.SCORE_PER_TEAM_FLIPFLOP_HOLD != 0)
		%this.tHoldThread = %game.schedule(%game.TIME_REQ_TEAM_HOLD_BONUS, "awardScoreTeamFFHold", %team, %this, true);

	%game.checkScoreLimit(%team);
}

function TAC_CnHGame::awardScoreTeamFFHold(%game, %team, %this, %msg)
{
	cancel(%this.tHoldThread);
   $TeamScore[%team] +=%game.SCORE_PER_TEAM_FLIPFLOP_HOLD;
	%sLimit = %game.getScoreLimit();
	if (%game.SCORE_PER_TEAM_FLIPFLOP_HOLD)
		messageAll('MsgCnHTeamCap', "", $TeamName[%team], %game.SCORE_PER_TEAM_FLIPFLOP_HOLD, %game.cleanWord(%this.name), %team, $teamScore[%team], %sLimit);

	// if either team's score is divisible by 100, send a console log message
	if(($TeamScore[%team] / 100) == (mFloor($TeamScore[%team] / 100)))
		for(%i = 1; %i <= %game.numTeams; %i++)
			logEcho("team "@%i@" score "@$TeamScore[%i]);

	%game.checkScoreLimit(%team);

	%this.tHoldThread = %game.schedule(%game.TIME_REQ_TEAM_HOLD_BONUS, "awardScoreTeamFFHold", %team, %this, false);
}

function TAC_CnHGame::startTimerPlayerFFCap(%game, %cl, %this)
{
	cancel(%this.pCapThread); //stop the last owner from collecting a cap bonus
	%this.pCapThread = %game.schedule(%game.TIME_REQ_PLYR_CAP_BONUS, "awardScorePlayerFFCap", %cl, %this);
	cancel(%this.tCapThread); //stop the old owners from collecting any cap bonus
	cancel(%this.tHoldThread); //stop the old owners from collecting any hold bonus
	%this.tCapThread = %game.schedule(%game.TIME_REQ_TEAM_CAP_BONUS, "awardScoreTeamFFCap", %cl.team, %this);
}

function TAC_CnHGame::startTimerTeamFFCap(%game, %team, %this, %time)
{
	%this.tCapThread = %game.schedule(%time, "awardScoreTeamFFCap", %team, %this);
}

//------------------------------------------------------------------------

function TAC_CnHGame::applyConcussion(%game, %player)
{
}
