// DisplayName = Convoy - Team Aerial Combat

//--- GAME RULES BEGIN ---
//Escort Jehrico to destination intact
//Game has two rounds: Round 1 ends when base is conquered or time runs out
//Round 2: Teams switch sides, play again
//If both Jehrico's are destroyed or both survive. Winner will be determined by points.
//Dont touch the ground...it hurts.
//--- GAME RULES END ---


//=====================================================================================
//	TAC2 - Convoy Mission Code								Wizard_TPG - 11/5/2001
//
//	This code includes:
//				Convoy Game Code
//				Player spawns with repair pack
//
//=====================================================================================

//Note For Mappers: Each team should have one spawn sphere that does NOT touch buildings.
//					This is where the jehrico will spawn......put it close to base though
//					for mounting.


//exec the AI scripts
exec("scripts/aiCTF.cs");


//-- tracking  ---
// .deaths .kills .genDefends .carrierKills .flagDefends .escortAssists .suicides .teamKills .turretKills .flagCaps
// .flagReturns .genDestroys .genRepairs

function TAC_ConvoyGame::initGameVars(%game)
{
	%game.SCORE_PER_VEHICLE_APC_KILL = 3;  	//enemy team gets this for killing your apc from a vehicle
	%game.SCORE_PER_OTHER_APC_KILL = 1;		//enemy team gets this for killing apc otherwise
	%game.SCORE_PER_OWN_APC_KILL = -3;		//Your team gets this if you tk an apc
	%game.SCORE_PER_BASE_APC_KILL = 0;		//Enemy team gets this if your veihcle is destroyed by enemy base turrets
	%game.SCORE_PER_TURRET_APC_KILL = 1;	//Enemy team gets this for kills by other turrets
	%game.SCORE_PER_CRASH_APC_KILL = -1;	//Your team gets this if you crash an apc
	%game.SCORE_PER_COLLIDE_APC_KILL = 1;	//Your team gets this if you collide with an enemy causing their vehicle to be destroyed
	%game.SCORE_PER_JEHRICO_KILL = 20000;
	%game.SCORE_PER_CONVOY_SUCCESS = 20000;

	%game.SCORE_PER_SUICIDE = 0;
	%game.SCORE_PER_TEAMKILL = 0;
	%game.SCORE_PER_DEATH = 0;

	%game.SCORE_PER_KILL = 0;
	%game.SCORE_PER_PLYR_FLAG_CAP = 0;
	%game.SCORE_PER_TEAM_FLAG_CAP = 10;
	%game.SCORE_PER_GEN_DESTROY = 0;
	%game.SCORE_PER_ESCORT_ASSIST = 0;

	%game.SCORE_PER_TURRET_KILL = 0;
	%game.SCORE_PER_FLAG_DEFEND = 0;
	%game.SCORE_PER_CARRIER_KILL = 0;
	%game.SCORE_PER_FLAG_RETURN = 0;
	%game.SCORE_PER_GEN_DEFEND = 0;
	%game.SCORE_PER_GEN_REPAIR = 0;

	%game.FLAG_RETURN_DELAY = 15 * 1000;

	%game.TIME_CONSIDERED_FLAGCARRIER_THREAT = 0;  //after damaging enemy flag carrier
	%game.RADIUS_GEN_DEFENSE = 0;  //meters
	%game.RADIUS_FLAG_DEFENSE = 0;  //meters

	%game.fadeTimeMS = 2000;
	%game.notifyMineDist = 7.5;
}

//--------------------------------------------------------------------------------------------------------------------

function TAC_ConvoyGame::missionLoadDone(%game)
{
   DefaultGame::missionLoadDone(%game);

   //clear the scores
   $teamScore[1] = 0;
   $teamScore[2] = 0;

   //decide which team is starting first
   if (getRandom() > 0.5)
   {
      %game.offenseTeam = 1;
      %defenseTeam = 2;
   }
   else
   {
      %game.offenseTeam = 2;
      %defenseTeam = 1;
   }

   //send the message
   messageAll('MsgSiegeStart', '\c2%1 is starting on offense', $teamName[%game.offenseTeam]);

   //if the first offense team is team2, switch the object team designation
   if (%game.offenseTeam == 2)
   {
      %group = nameToID("MissionGroup/Teams");
      %group.swapTeams();
      // search for vehicle pads also
      %mcg = nameToID("MissionCleanup");
      %mcg.swapVehiclePads();
   }

   //also ensure the objectives are all on the defending team
   %objSet = nameToId("MissionCleanup/Objectives");
   for(%j = 0; %j < %objSet.getCount(); %j++)
   {
      %obj = %objSet.getObject(%j);
      %obj.team = %defenseTeam;
      setTargetSensorGroup(%obj.getTarget(), %defenseTeam);
   }

   //indicate we're starting the game from the beginning...
   %game.firstHalf = true;
   %game.timeLimitMS = $Host::TimeLimit * 60 * 1000;
   %game.secondHalfCountDown = false;

   // save off turret bases' original barrels
   %game.checkTurretBases();

   MissionGroup.setupPositionMarkers(true);

}

function TAC_ConvoyGame::setUpTeams(%game)
{
	%game.numTeams = 2;
	DefaultGame::setUpTeams(%game);
	%team = 1;
   	%game.CreateJehrico(%team);
}

function TAC_ConvoyGame::CreateJehrico(%game, %team)
{
	%spawnLoc = %game.pickTeamSpawn(%team, true);
	%jehrico = TACMobileBaseVehicle.create(%team);
	for(%loc = 0;%loc<7;%loc++)
	{
		%locdata[%loc] = getword(%spawnLoc,%loc);
		if(%loc == 2)
			%locdata[%loc]+=10;
		if(%loc == 0)
			%outputloc = %locdata[%loc];
		else
			%outputloc = %outputloc @ " " @ %locdata[%loc];
	}
	%jehrico.setTransform( %outputloc );
	%jehrico.convoy = true;
}

function TAC_ConvoyGame::equip(%game, %player)
{
   for(%i =0; %i<$InventoryHudCount; %i++)
      %player.client.setInventoryHudItem($InventoryHudData[%i, itemDataName], 0, 1);

   %player.setInventory(RepairKit,1);
   %player.setInventory(FlareGrenade,6);
   %player.setInventory(Blaster,1);
   %player.setInventory(Chaingun,1);
   %player.setInventory(ChaingunAmmo, 100);
   %player.setInventory(Disc,1);
   %player.setInventory(DiscAmmo, 20);
   %player.setInventory(RepairPack,1);
   %player.setInventory(TargetingLaser, 1);
   %player.weaponCount = 3;
   %player.use("Blaster");
}

function TAC_ConvoyGame::VehicleKill(%game, %data, %obj)
{
	DefaultGame::VehicleKill(%game, %data, %obj);
}

function TAC_ConvoyGame::timeLimitReached(%game)
{
	echo("LOG: game over (timelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_ConvoyGame::scoreLimitReached(%game)
{
	echo("LOG: game over (scorelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_ConvoyGame::gameOver(%game)
{
	//send the winner message
	%winner = "";
	if ($teamScore[1] > $teamScore[2])
		%winner = $teamName[1];
	else if ($teamScore[2] > $teamScore[1])
		%winner = $teamName[2];

	if (%winner $= 'Storm')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.stowins.wav" );
	else if (%winner $= 'Inferno')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.infwins.wav" );
	else
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.gameover.wav" );

	//call the default
	DefaultGame::gameOver(%game);

   %game.sendDebriefing();
	messageAll('MsgClearObjHud', "");
	for(%i = 0; %i < ClientGroup.getCount(); %i ++)
	{
		%client = %group.getObject(%i);
		%game.resetScore(%client);
	}
   for(%j = 1; %j <= %game.numTeams; %j++)
		$TeamScore[%j] = 0;
}


////////////////////////////////////////////////////////////////////////////////////////
function TAC_ConvoyGame::clientMissionDropReady(%game, %client)
{
   messageClient(%client, 'MsgClientReady',"", %game.class);
	%game.resetScore(%client);
   for(%i = 1; %i <= %game.numTeams; %i++)
   {
      $Teams[%i].score = 0;
      messageClient(%client, 'MsgCTFAddTeam', "", %i, $TeamName[%i], $flagStatus[%i], $TeamScore[%i]);
   }

	messageClient(%client, 'MsgMissionDropInfo', '\c0You are in mission %1 (%2).', $MissionDisplayName, $MissionTypeDisplayName, $ServerName );

	DefaultGame::clientMissionDropReady(%game, %client);
}


function TAC_ConvoyGame::assignClientTeam(%game, %client, %respawn)
{
	DefaultGame::assignClientTeam(%game, %client, %respawn);
	// if player's team is not on top of objective hud, switch lines
	messageClient(%client, 'MsgCheckTeamLines', "", %client.team);
}

function TAC_ConvoyGame::updateKillScores(%game, %clVictim, %clKiller, %damageType, %implement)
{
	if(%clKiller <= 0 || %clKiller $= "")
	{
		%game.awardScoreDeath(%clVictim);
	}
	else if (%game.testKill(%clVictim, %clKiller)) //verify victim was an enemy
	{
		%game.awardScoreKill(%clKiller);
		%game.awardScoreDeath(%clVictim);
	}
	else
	{
		if (%game.testSuicide(%clVictim, %clKiller, %damageType))  //otherwise test for suicide
		{
			%game.awardScoreSuicide(%clVictim);
		}
		else
		{
			if (%game.testTeamKill(%clVictim, %clKiller)) //otherwise test for a teamkill
				%game.awardScoreTeamKill(%clVictim, %clKiller);
		}
	}
}

function TAC_ConvoyGame::checkScoreLimit(%game, %team)
{
	%scoreLimit = 10000;
   	if($TeamScore[%team] >= %scoreLimit)
      	%game.scoreLimitReached();
}

function TAC_ConvoyGame::enterMissionArea(%game, %playerData, %player)
{
   	%player.client.outOfBounds = false;
   	messageClient(%player.client, 'EnterMissionArea', '\c1You are back in the mission area.');
	//the instant a player leaves the mission boundary, the flag is dropped, and the return is scheduled...
	if (%player.holdingFlag > 0)
	{
		cancel($FlagReturnTimer[%player.holdingFlag]);
		$FlagReturnTimer[%player.holdingFlag] = "";
	}
}

function TAC_ConvoyGame::leaveMissionArea(%game, %playerData, %player)
{
	// maybe we'll do this just in case
   	%player.client.outOfBounds = true;
	// if the player is holding a flag, strip it and throw it back into the mission area
	// otherwise, just print a message
	if(%player.holdingFlag > 0)
		%game.boundaryLoseFlag(%player);
	else
      messageClient(%player.client, 'MsgLeaveMissionArea', '\c1You have left the mission area.');
	echo("LOG: "@%player@" ("@%player.client@") leave mission area");
}

function TAC_ConvoyGame::pickTeamSpawn(%game, %team, %jehrico)
{
	if(%jehrico)
	{
		%sphere = %game.selectSpawnSphere(%team);
	   	if (%sphere !=-1)
	   	{
			%zone = %game.selectSpawnZone(%sphere);
			%area = "outdoor";
      		%radius = %sphere.radius;
      		%sphereTrans = %sphere.getTransform();
      		%sphereCtr = getWord(%sphereTrans, 0) @ " " @ getWord(%sphereTrans, 1) @ " " @ getWord(%sphereTrans, 2);   //don't need full transform here, just x, y, z
			if (navGraphExists())
			{
				%avoidThese = $TypeMasks::VehicleObjectType  | $TypeMasks::MoveableObjectType |
						   $TypeMasks::PlayerObjectType   | $TypeMasks::TurretObjectType |
						   $TypeMasks::InteriorObjectType;

         		for (%tries = 0; %tries < 10; %tries++)
         		{
            		%nodeIndex = navGraph.randNode(%sphereCtr, %radius, %useIndoor, %useOutdoor);
            		if (%nodeIndex >= 0)
            		{
               			%loc = navGraph.randNodeLoc(%nodeIndex);
               			%adjUp = VectorAdd(%loc, "0 0 1.0");   // don't go much below

               			if (ContainerBoxEmpty( %avoidThese, %adjUp, 2.0))
                  		break;
            		}
         		}

         		if (%nodeIndex >= 0)
         		{
           			%loc = navGraph.randNodeLoc(%nodeIndex);
            		if (%zone)
            		{
               			%trns = %loc @ " 0 0 1 0";
               			%spawnLoc = whereToLook(%trns);
            		}
            		else
            		{
               			%rot = %game.selectSpawnFacing(%loc, %sphereCtr, %zone);
               			%spawnLoc = %loc @ %rot;
            		}
            		return %spawnLoc;
         		}
         		else
         		{
            		echo( "randLoc() didn't find an " @ %area @ " node.  Try readjusting the weights or increasing your sphere radius." );
            		return -1;
         		}
      		}
        	else
			{
				echo("No navigation graph is present.  Build one.");
				return -1;
			}
		}
		else
		{
			echo("No spawn spheres found for team " @ %team);
			return -1;
		}
	}
	else
		DefaultGame::pickTeamSpawn(%game, %team);
}

function TAC_ConvoyGame::onEnterTrigger(%data, %obj, %colObj)
{
	if(%colObj.convoy)
	{
		%teampoints = %game.SCORE_PER_CONVOY_SUCCESS;
		$TeamScore[%colObj.team] += %teampoints;
		%game.checkScoreLimit(%colObj.team);
		messageAll('msgConvoySuccess', '\c0The Convoy Jehrico made it to the destination!');
	}
}

function TAC_ConvoyGame::VehicleKill(%game, %data, %obj)
{
	Parent::VehicleKill(%game, %data, %obj);
   	if ( %obj.getDataBlock().className $= WheeledVehicleData && %obj.convoy)
   	{
		messageAll('msgConvoyDestroyed', '\c0The Convoy Jehrico was destroyed!');
		%teampoints = %game.SCORE_PER_JEHRICO_KILL;
		%scoreteam = %obj.team == 1 ? 2 : 1;
		$TeamScore[%scoreteam] += %teampoints;
		%game.checkScoreLimit(%scoreteam);
		return;
	}
}
