// DisplayName = Team Deathmatch - Team Aerial Combat

//--- GAME RULES BEGIN ---
//Destroy as many enemy vehicles without losing your own
//Score 50 points for killing enemy vehicles from a vehicle
//Score 20 point for killing enemy vehicles otherwise
//Lose 50 points for destroying own team vehicles.
//Keep a manned vehicle in mission area at all times or your team will lose one point every second.
//--- GAME RULES END ---


//=====================================================================================
//	TAC2 - TDM Mission Code									Wizard_TPG - 11/5/2001
//
//	This code includes:
//				Basic TDM Game Code
//				Player spawns with repair pack
//
//=====================================================================================


//exec the AI scripts
exec("scripts/aiCTF.cs");


package TAC_TDMGame
{
	//Remove Hawkeye and MPB from invs in TDM
	function StationVehiclePad::onAdd(%this, %obj)
	{
	   	Parent::onAdd(%this, %obj);

		%sv = %obj.station;
		%sv.vehicle[TACMobileBaseVehicle] = false;
		%sv.vehicle[TACScoutFlyer] = false;
	}
};



//-- tracking  ---
// .deaths .kills .genDefends .carrierKills .flagDefends .escortAssists .suicides .teamKills .turretKills .flagCaps
// .flagReturns .genDestroys .genRepairs

function TAC_TDMGame::initGameVars(%game)
{
	%game.SCORE_PER_VEHICLE_APC_KILL = 50;  	//enemy team gets this for killing your apc from a vehicle
	%game.SCORE_PER_OTHER_APC_KILL = 20;		//enemy team gets this for killing apc otherwise
	%game.SCORE_PER_OWN_APC_KILL = -50;		//Your team gets this if you tk an apc
	%game.SCORE_PER_BASE_APC_KILL = 0;		//Enemy team gets this if your veihcle is destroyed by enemy base turrets
	%game.SCORE_PER_TURRET_APC_KILL = 0;	//Enemy team gets this for kills by other turrets
	%game.SCORE_PER_CRASH_APC_KILL = -50;	//Your team gets this if you crash an apc
	%game.SCORE_PER_COLLIDE_APC_KILL = 50;	//Your team gets this if you collide with an enemy causing their vehicle to be destroyed
	%game.SCORE_DEDUCTION_NO_VEHICLE_INFIELD = -1;  //The amount that gets deducted per second that a manned vehicle is not in playing field

	%game.SCORE_PER_SUICIDE = 0;
	%game.SCORE_PER_TEAMKILL = 0;
	%game.SCORE_PER_DEATH = 0;

	%game.SCORE_PER_KILL = 0;
	%game.SCORE_PER_PLYR_FLAG_CAP = 0;
	%game.SCORE_PER_TEAM_FLAG_CAP = 0;
	%game.SCORE_PER_GEN_DESTROY = 0;
	%game.SCORE_PER_ESCORT_ASSIST = 0;

	%game.SCORE_PER_TURRET_KILL = 0;
	%game.SCORE_PER_FLAG_DEFEND = 0;
	%game.SCORE_PER_CARRIER_KILL = 0;
	%game.SCORE_PER_FLAG_RETURN = 0;
	%game.SCORE_PER_GEN_DEFEND = 0;
	%game.SCORE_PER_GEN_REPAIR = 0;

	%game.FLAG_RETURN_DELAY = 15 * 1000;

	%game.TIME_CONSIDERED_FLAGCARRIER_THREAT = 0;  //after damaging enemy flag carrier
	%game.RADIUS_GEN_DEFENSE = 0;  //meters
	%game.RADIUS_FLAG_DEFENSE = 0;  //meters

	%game.fadeTimeMS = 2000;
	%game.notifyMineDist = 7.5;
}

//--------------------------------------------------------------------------------------------------------------------

function TAC_TDMGame::missionLoadDone(%game)
{
	%game.numTeams = 2;

   //continue with the default function
   DefaultGame::missionLoadDone(%game);
}

function TAC_TDMGame::setUpTeams(%game)
{
   DefaultGame::setUpTeams(%game);
   %game.numTeams = 2;
   setSensorGroupCount(%game.numTeams + 1);
   %game.TDMTeam1PointCheck = %game.schedule(1000, "TDMTeamPointCheck", 1);
   %game.TDMTeam2PointCheck = %game.schedule(1000, "TDMTeamPointCheck", 2);
}

function TAC_TDMGame::equip(%game, %player)
{
   for(%i =0; %i<$InventoryHudCount; %i++)
      %player.client.setInventoryHudItem($InventoryHudData[%i, itemDataName], 0, 1);

   %player.setInventory(RepairKit,1);
   %player.setInventory(FlareGrenade,6);
   %player.setInventory(Blaster,1);
   %player.setInventory(Chaingun,1);
   %player.setInventory(ChaingunAmmo, 100);
   %player.setInventory(Disc,1);
   %player.setInventory(DiscAmmo, 20);
   %player.setInventory(RepairPack,1);
   %player.setInventory(TargetingLaser, 1);
   %player.weaponCount = 3;
   %player.use("Blaster");
}

function TAC_TDMGame::VehicleKill(%game, %data, %obj)
{
	DefaultGame::VehicleKill(%game, %data, %obj);
}

function TAC_TDMGame::timeLimitReached(%game)
{
	echo("LOG: game over (timelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_TDMGame::scoreLimitReached(%game)
{
	echo("LOG: game over (scorelimit)");
	%game.gameOver();
   schedule( 5000, %game, cycleMissions );
}

function TAC_TDMGame::gameOver(%game)
{
	//send the winner message
	%winner = "";
	if ($teamScore[1] > $teamScore[2])
		%winner = $teamName[1];
	else if ($teamScore[2] > $teamScore[1])
		%winner = $teamName[2];

	if (%winner $= 'Storm')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.stowins.wav" );
	else if (%winner $= 'Inferno')
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.infwins.wav" );
	else
	   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.gameover.wav" );

	//call the default
	DefaultGame::gameOver(%game);

   %game.sendDebriefing();
	messageAll('MsgClearObjHud', "");
	for(%i = 0; %i < ClientGroup.getCount(); %i ++)
	{
		%client = %group.getObject(%i);
		%game.resetScore(%client);
	}
   for(%j = 1; %j <= %game.numTeams; %j++)
		$TeamScore[%j] = 0;
}


////////////////////////////////////////////////////////////////////////////////////////
function TAC_TDMGame::clientMissionDropReady(%game, %client)
{
   	messageClient(%client, 'MsgClientReady',"", %game.class);
   	for(%i = 1; %i <= %game.numTeams; %i++)
   	{
      	messageClient(%client, 'MsgTDMAddTeam', "", %i, $TeamName[%i], $TeamScore[%i]);
   	}

	messageClient(%client, 'MsgMissionDropInfo', '\c0You are in mission %1 (%2).', $MissionDisplayName, $MissionTypeDisplayName, $ServerName );

	DefaultGame::clientMissionDropReady(%game, %client);
}


function TAC_TDMGame::assignClientTeam(%game, %client, %respawn)
{
	DefaultGame::assignClientTeam(%game, %client, %respawn);
	// if player's team is not on top of objective hud, switch lines
	messageClient(%client, 'MsgCheckTeamLines', "", %client.team);
}

function TAC_TDMGame::updateKillScores(%game, %clVictim, %clKiller, %damageType, %implement)
{
	if(%clKiller <= 0 || %clKiller $= "")
	{
		%game.awardScoreDeath(%clVictim);
	}
	else if (%game.testKill(%clVictim, %clKiller)) //verify victim was an enemy
	{
		%game.awardScoreKill(%clKiller);
		%game.awardScoreDeath(%clVictim);
	}
	else
	{
		if (%game.testSuicide(%clVictim, %clKiller, %damageType))  //otherwise test for suicide
		{
			%game.awardScoreSuicide(%clVictim);
		}
		else
		{
			if (%game.testTeamKill(%clVictim, %clKiller)) //otherwise test for a teamkill
				%game.awardScoreTeamKill(%clVictim, %clKiller);
		}
	}
}

function TAC_TDMGame::TDMTeamPointCheck(%game, %team)
{
	if(%team == 1)
		cancel(%game.TDMTeam1PointCheck);
	else
		cancel(%game.TDMTeam2PointCheck);

	//Does this team have a client in a vehicle in the mission area?
	%foundvehicle = false;
	for(%i = 0; %i < ClientGroup.getCount(); %i ++)
	{
		%client = %group.getObject(%i);
		if(%client.team == %team && !%client.outOfBounds)
		{
			%player = %client.player;
			%vehicle = %player.getObjectMount();
			if(isObject(%vehicle))
				%foundvehicle = true;
		}
		if(%foundvehicle)
			%i = ClientGroup.getCount();
	}

	if($TeamScore[%team] > 0 && !%foundvehicle)
	{
		%deduction = %game.SCORE_DEDUCTION_NO_VEHICLE_INFIELD !$= "" ? %game.SCORE_DEDUCTION_NO_VEHICLE_INFIELD : 0;
		$TeamScore[%team] += %deduction;
		%enemyteam = %team == 1 ? 2 : 1;
		%points = %deduction * -1;
		%plural = %points > 1 ? "s" : "";
		if(%game.firstPointLossMessage)
		{
			messageTeam(%team, 'MsgTACTDMPointsLoss', '\c0Your team is losing points because there is no mounted vehicle in the mission area!~wfx/misc/TDM_Losing_Points_Alarm.wav', $TeamScore[%team], %team);
			messageTeam(%enemyteam, 'MsgTACTDMPointsLoss', '\c0Enemy team is losing points for having no mounted vehicle in the mission area!', $TeamScore[%team], %team);
			messageTeam(0, 'MsgTACTDMPointsLoss', '\c0 %3 is losing points for having no mounted vehicle in the mission area!', $TeamScore[%team], %team, $TeamName[%team]);
		}
		%game.checkScoreLimit(%team);
	}

	if(%team == 1)
	  	%game.TDMTeam1PointCheck = %game.schedule(1000, "TDMTeamPointCheck", 1);
	else
   		%game.TDMTeam2PointCheck = %game.schedule(1000, "TDMTeamPointCheck", 2);
}

function TAC_TDMGame::checkScoreLimit(%game, %team)
{
	echo("*** current score = ",$TeamScore[1]," - ",$TeamScore[2]);
	%scoreLimit = MissionGroup.TAC_TDM_scoreLimit;
	// default of 50 if scoreLimit not defined
	if(%scoreLimit $= "")
		%scoreLimit = 500;
   if($TeamScore[%team] >= %scoreLimit)
      %game.scoreLimitReached();
}

function TAC_TDMGame::enterMissionArea(%game, %playerData, %player)
{
	if(%player.getState() $= "Dead")
	return;

   	%player.client.outOfBounds = false;
   	messageClient(%player.client, 'EnterMissionArea', '\c1You are back in the mission area.');
	//the instant a player leaves the mission boundary, the flag is dropped, and the return is scheduled...
	if (%player.holdingFlag > 0)
	{
		cancel($FlagReturnTimer[%player.holdingFlag]);
		$FlagReturnTimer[%player.holdingFlag] = "";
	}
}

function TAC_TDMGame::leaveMissionArea(%game, %playerData, %player)
{
	if(%player.getState() $= "Dead")
	return;

	// maybe we'll do this just in case
   	%player.client.outOfBounds = true;
	// if the player is holding a flag, strip it and throw it back into the mission area
	// otherwise, just print a message
	if(%player.holdingFlag > 0)
		%game.boundaryLoseFlag(%player);
	else
      messageClient(%player.client, 'MsgLeaveMissionArea', '\c1You have left the mission area.');
	echo("LOG: "@%player@" ("@%player.client@") leave mission area");
}

function TAC_TDMGame::applyConcussion(%game, %player)
{
   %game.dropFlag( %player );
}

